@@ -675,6 +675,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(object_instance)
             GParameter *params;
             int n_params;
             GTypeQuery query;
+            JSObject *old_jsobj;
 
             gtype = g_registered_type_info_get_g_type( (GIRegisteredTypeInfo*) priv->info);
             if (gtype == G_TYPE_NONE) {
@@ -693,6 +694,23 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(object_instance)
             priv->gobj = g_object_newv(gtype, n_params, params);
             free_g_params(params, n_params);
 
+            old_jsobj = peek_js_obj(context, priv->gobj);
+            if (old_jsobj != NULL) {
+                /* g_object_newv returned an object that's already tracked by a JS
+                 * object. Let's assume this is a singleton like IBus.IBus and return
+                 * the existing JS wrapper object.
+                 *
+                 * 'object' has a value that was originally created by
+                 * JS_NewObjectForConstructor in GJS_NATIVE_CONSTRUCTOR_PRELUDE, but
+                 * we're not actually using it, so just let it get collected. Avoiding
+                 * this would require a non-trivial amount of work.
+                 * */
+                object = old_jsobj;
+                g_object_unref(priv->gobj); /* We already own a reference */
+                priv->gobj = NULL;
+                goto out;
+            }
+
             g_type_query(gtype, &query);
             JS_updateMallocCounter(context, query.instance_size);
 
@@ -715,9 +733,9 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(object_instance)
             unthreadsafe_template_for_constructor.gobj = NULL;
 
             g_object_ref_sink(priv->gobj);
+            g_assert(peek_js_obj(context, priv->gobj) == NULL);
         }
 
-        g_assert(peek_js_obj(context, priv->gobj) == NULL);
         set_js_obj(context, priv->gobj, object);
 
 #if DEBUG_DISPOSE
@@ -760,6 +778,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(object_instance)
                                     g_base_info_get_name ( (GIBaseInfo*) priv->info) ));
     }
 
+ out:
     GJS_NATIVE_CONSTRUCTOR_FINISH(object_instance);
 
     return JS_TRUE;
@@ -1522,9 +1541,13 @@ gjs_object_from_g_object(JSContext    *context,
         obj = gjs_construct_object_dynamic(context, proto,
                                               0, NULL);
 
+        unthreadsafe_template_for_constructor.info = NULL;
+        unthreadsafe_template_for_constructor.gobj = NULL;
+
         g_base_info_unref( (GIBaseInfo*) info);
 
-        g_assert(peek_js_obj(context, gobj) == obj);
+        if (obj != NULL)
+            g_assert(peek_js_obj(context, gobj) == obj);
     }
 
     return obj;
